"""
Formalisms related to heat transfers.
"""
import math

from . import raw


def heat_conductance(hc, ws):
    """Resistance to heat transfer between canopy and atmosphere.

    Weather station is supposed to be at 2m height.

    References: eq3, 4, 5 and 6

    Args:
        hc (float): [m] canopy height
        ws (float): [m.s-1] wind speed at 2m high

    Returns:
        (float): [m.s-1] heat conductance
    """
    zm = zh = 2  # [m] weather station altitude
    zom = raw.eq4(hc)
    zoh = raw.eq5(hc)
    d = raw.eq6(hc)

    # don't use eq3 in the end to avoid numerical instabilities when ws=0
    # ra = raw.eq3(zm, zh, d, zom, zoh, ws)

    return raw.von_karman ** 2 * ws / (math.log((zm - d) / zom) * math.log((zh - d) / zoh))



# {# pkglts, glabpkg_dev
import allen1989


def test_package_exists():
    assert allen1989.__version__

# #}
# {# pkglts, glabdata, after glabpkg_dev

def test_paths_are_valid():
    assert allen1989.pth_clean.exists()
    try:
        assert allen1989.pth_raw.exists()
    except AttributeError:
        pass  # package not installed in editable mode

# #}



"""
Raw formalisms as written in paper
"""
import math

von_karman = 0.41  # [-] https://en.wikipedia.org/wiki/Von_K%C3%A1rm%C3%A1n_constant
r1_alfalfa = 100  # [s.m-1] average minimum daytime value of stomatal resistance for a single leaf


def eq3(zm, zh, d, zom, zoh, uz):
    return math.log((zm - d) / zom) * math.log((zh - d) / zoh) / (von_karman ** 2 * uz)


def eq4(hc):
    return 0.123 * hc


def eq5(hc):
    return 0.0123 * hc


def eq6(hc):
    return 0.67 * hc


def eq7(r1, lai):
    return r1 / (0.5 * lai)


def eq8(hc):
    return 24 * hc


def eq9(hc):
    assert hc > 0.03
    return 5.5 + 1.5 * math.log(hc)



import pytest

from allen1989 import raw
from allen1989.transfer import heat_conductance


def test_heat_conductance_corresponds_to_raw():
    hc = 1.5  # [m]
    zm = zh = 2  # [m] weather station altitude
    zom = raw.eq4(hc)
    zoh = raw.eq5(hc)
    d = raw.eq6(hc)

    for ws in (0.3, 1, 2, 10):  # [m.s-1]
        ra = raw.eq3(zm, zh, d, zom, zoh, ws)
        assert heat_conductance(hc, ws) == pytest.approx(1 / ra, abs=1e-15)



"""
Formalisms from Allen at al. 1989 paper
"""
# {# pkglts, src
# FYEO
# #}
# {# pkglts, version, after src
from . import version

__version__ = version.__version__
# #}
# {# pkglts, glabdata, after version
from .info import *
# #}



"""
File that regroup all access points visible from the outside world.
"""
from pathlib import Path

__all__ = ["pth_clean"]

pth_clean = Path(__file__).parent / "clean"

if Path(__file__).parent.parent.name == "src":  # provided for convenience only when installed in editable
    __all__.append("pth_raw")
    pth_raw = Path(__file__).parent.parent.parent / "raw"
